/*--------------------------------------------------------------------------*/
/* A9101.H                                                                  */
/*                                                                          */
/* Header file for A9101 RF SoC microcontroller.                            */
/* Copyright (c) 2013 AMICCOM Electronics Corp.                             */
/*                                                                          */
/* All rights reserved.                                                     */
/*--------------------------------------------------------------------------*/
#ifndef __A9101_H__
#define __A9101_H__
//--------------------------------------------------------------------------
// 8051 SFR
//--------------------------------------------------------------------------

/*  BYTE Registers  */
sfr P0      = 0x80;
sfr P1      = 0x90;
sfr P2      = 0xA0;
sfr P3      = 0xB0;
sfr PSW     = 0xD0;
sfr ACC     = 0xE0;
sfr B       = 0xF0;
sfr SP      = 0x81;
sfr DPL0     = 0x82;
sfr DPH0     = 0x83;
sfr DPL1    = 0x84;
sfr DPH1    = 0x85;
sfr DPS     = 0x86; /* Data pointers select      */
sfr PCON    = 0x87;
sfr TCON    = 0x88;
sfr TMOD    = 0x89;
sfr TL0     = 0x8A;
sfr TL1     = 0x8B;
sfr TH0     = 0x8C;
sfr TH1     = 0x8D;
sfr CKCON   = 0x8E;
sfr EIF     = 0x91;
sfr SCON    = 0x98;
sfr SBUF    = 0x99;
sfr FLASHCTRL = 0x9A;
sfr FLASHMR = 0x9B;
sfr IE      = 0xA8;
sfr PWM0CON = 0xA9;
sfr PWM0H   = 0xAA;
sfr PWM0L   = 0xAB;
sfr PWM1CON = 0xB1;
sfr PWM1H   = 0xB2;
sfr PWM1L   = 0xB3;
sfr IP      = 0xB8;
sfr PCONE   = 0xB9;
sfr RSFLAG  = 0xBA;
sfr IOSEL   = 0xBB;
sfr T2CON   = 0xC8;
sfr T2IF    = 0xC9;
sfr RCAP2L  = 0xCA;
sfr RCAP2H  = 0xCB;
sfr TL2     = 0xCC;
sfr TH2     = 0xCD;
sfr WDCON   = 0xD8;
sfr EIE     = 0xE8;
sfr TA      = 0xEB;
sfr SPCR    = 0xEC;
sfr SPSR    = 0xED;
sfr SPDR    = 0xEE;
sfr SSCR    = 0xEF;
sfr I2CSADR = 0xF1;
sfr I2CSCR  = 0xF2;
sfr I2CSBUF = 0xF3;
sfr I2CMSA  = 0xF4;
sfr I2CMCR  = 0xF5;
sfr I2CMBUF = 0xF6;
sfr I2CMTP  = 0xF7;
sfr EIP     = 0xF8;

sfr P0OE    = 0xD1;
sfr P0PUN   = 0xD2;
sfr P0WUN   = 0xD3;
sfr P1OE    = 0xD9;
sfr P1PUN   = 0xDA;
sfr P1WUN   = 0xDB;
sfr P3OE    = 0xE1;
sfr P3PUN   = 0xE2;
sfr P3WUN   = 0xE3;

/*  BIT Registers  */

sbit ERFINT = EIE ^ 3;
sbit EWDI = EIE ^ 5;
sbit PWDI = EIP ^ 5;

/*  PSW  */
sbit CY     = PSW ^ 7;
sbit AC     = PSW ^ 6;
sbit F0     = PSW ^ 5;
sbit RS1    = PSW ^ 4;
sbit RS0    = PSW ^ 3;
sbit OV     = PSW ^ 2;
sbit P      = PSW ^ 0;

/*  TCON  */
sbit TF1    = TCON ^ 7;
sbit TR1    = TCON ^ 6;
sbit TF0    = TCON ^ 5;
sbit TR0    = TCON ^ 4;
sbit IE1    = TCON ^ 3;
sbit IT1    = TCON ^ 2;
sbit IE0    = TCON ^ 1;
sbit IT0    = TCON ^ 0;

/*  IE  */
sbit EA     = IE ^ 7;
sbit ET2    = IE ^ 5;
sbit ES     = IE ^ 4;
sbit ET1    = IE ^ 3;
sbit EX1    = IE ^ 2;
sbit ET0    = IE ^ 1;
sbit EX0    = IE ^ 0;

/*  IP  */
sbit PT2    = IP ^ 5;
sbit PS     = IP ^ 4;
sbit PT1    = IP ^ 3;
sbit PX1    = IP ^ 2;
sbit PT0    = IP ^ 1;
sbit PX0    = IP ^ 0;

/*  P3  */
sbit RD     = P3 ^ 7;
sbit WR     = P3 ^ 6;
sbit T1     = P3 ^ 5;
sbit T0     = P3 ^ 4;
sbit INT1   = P3 ^ 3;
sbit INT0   = P3 ^ 2;
sbit TXD    = P3 ^ 1;
sbit RXD    = P3 ^ 0;

/*  SCON  */
sbit SM0    = SCON ^ 7;
sbit SM1    = SCON ^ 6;
sbit SM2    = SCON ^ 5;
sbit REN    = SCON ^ 4;
sbit TB8    = SCON ^ 3;
sbit RB8    = SCON ^ 2;
sbit TI     = SCON ^ 1;
sbit RI     = SCON ^ 0;

/*  P1  */
sbit T2EX   = P1 ^ 1;
sbit T2     = P1 ^ 0;

/*  T2CON  */
sbit TF2    = T2CON ^ 7;
sbit EXF2   = T2CON ^ 6;
sbit RCLK   = T2CON ^ 5;
sbit TCLK   = T2CON ^ 4;
sbit EXEN2  = T2CON ^ 3;
sbit TR2    = T2CON ^ 2;
sbit C_T2   = T2CON ^ 1;
sbit CP_RL2 = T2CON ^ 0;

/*  P0  */
sbit P0_7   = P0 ^ 7;
sbit P0_6   = P0 ^ 6;
sbit P0_5   = P0 ^ 5;
sbit P0_4   = P0 ^ 4;
sbit P0_3   = P0 ^ 3;
sbit P0_2   = P0 ^ 2;
sbit P0_1   = P0 ^ 1;
sbit P0_0   = P0 ^ 0;

/*  P1  */
sbit P1_7   = P1 ^ 7;
sbit P1_6   = P1 ^ 6;
sbit P1_5   = P1 ^ 5;
sbit P1_4   = P1 ^ 4;
sbit P1_3   = P1 ^ 3;
sbit P1_2   = P1 ^ 2;
sbit P1_1   = P1 ^ 1;
sbit P1_0   = P1 ^ 0;

/*  P2  */
sbit P2_7   = P2 ^ 7;
sbit P2_6   = P2 ^ 6;
sbit P2_5   = P2 ^ 5;
sbit P2_4   = P2 ^ 4;
sbit P2_3   = P2 ^ 3;
sbit P2_2   = P2 ^ 2;
sbit P2_1   = P2 ^ 1;
sbit P2_0   = P2 ^ 0;

/*  P3  */
sbit P3_7   = P3 ^ 7;
sbit P3_6   = P3 ^ 6;
sbit P3_5   = P3 ^ 5;
sbit P3_4   = P3 ^ 4;
sbit P3_3   = P3 ^ 3;
sbit P3_2   = P3 ^ 2;
sbit P3_1   = P3 ^ 1;
sbit P3_0   = P3 ^ 0;


//--------------------------------------------------------------------------
// RF Register
//--------------------------------------------------------------------------
#define CONFIG            0x0800
#define MODE_REG          0x0800
#define MODEC1_REG        0x0801
#define MODEC2_REG        0x0802
#define CALC_REG          0x0803
#define FIFO1_REG         0x0804
#define FIFO2_REG         0x0805
#define Clock_REG         0x0806
#define VCB_REG           0x0807
#define RCOSC3_REG        0x0808
#define CKO_REG           0x0809
#define GIO1_REG          0x080A
#define GIO2_REG          0x080B
#define DATARATECLK_REG   0x080C
#define PLL1_REG          0x080D
#define PLL2_REG          0x080E
#define PLL3_REG          0x080F
#define PLL4_REG          0x0810
#define PLL5_REG          0x0811
#define CHGROUP1_REG      0x0812
#define CHGROUP2_REG      0x0813
#define TX1_REG           0x0814
#define TX2_REG           0x0815
#define DELAY1_REG        0x0816
#define DELAY2_REG        0x0817
#define RX_REG            0x0818
#define RXGAIN1_REG       0x0819
#define RXGAIN2_REG       0x081A
#define RXGAIN3_REG       0x081B
#define RXGAIN4_REG       0x081C
#define RSSI_REG          0x081D
#define ADC_REG           0x081E
#define CODE1_REG         0x081F
#define CODE2_REG         0x0820
#define CODE3_REG         0x0821
#define IFCAL1_REG        0x0822
#define IFCAL2_REG        0x0823
#define VCOC_REG          0x0824
#define VCOBAND1_REG      0x0825
#define VCOBAND2_REG      0x0826
#define VCODEV1_REG       0x0827
#define VCODEV2_REG       0x0828
#define VCODEV3_REG       0x0829
#define VCOMODDELAY_REG   0x082A
#define BATTERY_REG       0x082B
#define TXTEST_REG        0x082C
#define A9101_RXDEM1_REG  0x082D
#define A9101_RXDEM2_REG  0x082E
#define CPC1_REG          0x082F
#define CPC2_REG          0x0830
#define CRYSTAL_REG       0x0831
#define PLLTEST_REG       0x0832
#define VCOTEST_REG       0x0833
#define RFANALOG_REG      0x0834
#define IFAT_REG          0x0835
#define CHSELECT_REG      0x0836
#define VRB_REG           0x0837
#define DataRate_REG      0x0838
#define FCR_REG           0x0839
#define ARD_REG           0x083A
#define AFEP_REG          0x083B
#define WMUX_REG          0x083C
#define FCB0_REG          0x083D
#define FCB1_REG          0x083E
#define FCB2_REG          0x083F
#define FCB3_REG          0x0840
#define ID7_REG           0x0841
#define ID6_REG           0x0842
#define ID5_REG           0x0843
#define ID4_REG           0x0844
#define ID3_REG           0x0845
#define ID2_REG           0x0846
#define ID1_REG           0x0847
#define ID0_REG           0x0848
#define DID3_REG          0x0849
#define DID2_REG          0x084A
#define DID1_REG          0x084B
#define DID0_REG          0x084C
#define ADCCTL_REG        0x084D
#define ADCAVG1_REG       0x084E
#define ADCAVG2_REG       0x084F
#define ADCAVG3_REG       0x0850
#define TMRINV_REG        0x0851
#define TMRCTL_REG        0x0852
#define EXT1_REG          0x0853
#define EXT2_REG          0x0854
#define EXT3_REG          0x0855
#define EXT4_REG          0x0856
#define EXT5_REG          0x0857
#define PWRCTL_REG        0x0858
#define INTSW_REG         0x0859
#define RCOSC4_REG        0x085A
#define RCOSC5_REG        0x085B
#define RCOSC6_REG        0x085C
#define RCOSC7_REG        0x085D
#define RCOSC8_REG        0x085E
#define RTC_REG           0x085F
#define AGC1_REG          0x0860
#define AGC2_REG          0x0861
#define AGC3_REG          0x0862
#define FLASH_REG         0x0863
#define TX_FIFO           0x0900  // 0x0900~0x097F = 128 bytes
#define RX_FIFO           0x0980  // 0x0980~0x09FF = 128 bytes
//--------------------------------------------------------------------------
// CODEC Register
//--------------------------------------------------------------------------
#define CODEC_ANACTL    0x0A00
#define CODEC_AGCCAL    0x0A01
#define CODEC_AMP       0x0A02
#define CODEC_PKCAL     0x0A03
#define CODEC_OSCAL1    0x0A04
#define CODEC_OSCAL2    0x0A05
#define CODEC_ADCDLY    0x0A06
#define CODEC_VOL       0x0A08
#define CODEC_SRSEL     0x0A09
#define CODEC_STATE     0x0A0A
#define CODEC_ADCRB     0x0A0B
#define CODEC_DACWB     0x0A0C
#define CODEC_CODECTL   0x0A0E
#define CODEC_EFFCTL    0x0A0D
#define CODEC_TESTMODE  0x0A0F
#define CODEC_FSTEPL    0x0A10
#define CODEC_FSTEPH    0x0A11
//--------------------------------------------------------------------------
// Strobe and Reset Command
//--------------------------------------------------------------------------

/* RF Strobe command */
#define CMD_SLEEP       0x80    // SLEEP mode
#define CMD_IDLE        0x90    // IDLE mode
#define CMD_STBY        0xA0    // Standby mode
#define CMD_PLL         0xB0    // PLL mode
#define CMD_RX          0xC0    // RX mode
#define CMD_TX          0xD0    // TX mode

/* RF Reset command */
#define RF_RST          0x80    //RF reset
#define TXPOINT_RST     0x40    //TX FIFO address pointrt reset
#define RXPOINT_RST     0x20    //RX FIFO address pointer reset
#define FIFODATA_RST    0x10

/*********************************************************************
**  ANACTL Analog control register (Address: 0A00h)       **
*********************************************************************/
#define PDN_CAB 0x40 // power down class AB amplifier, active low.
#define PDN_DA  0x20 // power down analog DAC, active low.

//  AC filter's input.
#define PDN_MIC 0x08 // power down microphone amplifier, active low.
#define PDN_AGC 0x04 // power down AGC, active low.
#define PDN_PK  0x02 // power down peak detection, active low.
#define PDN_AD  0x01 // power down analog ADC, active low.

/*********************************************************************
**  AGCCAL AGC Calibration register (Address: 0A01h)        **
*********************************************************************/
// VOX and AGC threshold setting regarding to input signal peak level.
// VOXS: VOX threshold.
#define VOXS_50mv   0x00
#define VOXS_100mv  0x20
#define VOXS_200mv  0x40
#define VOXS_300mv  0x60
#define VOXS_400mv  0x80


//  RS_DL AGC peak detect charge time: 10ms.
// [00]: 10ms. [01]: 20ms. [10]: 40ms. [11]: 80ms.
#define RS_DL_100us     0x00
#define RS_DL_1ms       0x04
#define RS_DL_10ms      0x08
#define RS_DL_20ms      0x0C
#define RS_DL_40ms      0x10
#define RS_DL_80ms      0x14
#define RS_DL_100ms     0x18
#define RS_DL_1000ms    0x1C

// AGC measurement delay time.
//[00]: 4ms. [01]: 8ms. [10]: 12ms. [11]: 16ms.
#define AGC_DL_4ms      0x00
#define AGC_DL_8ms      0x01
#define AGC_DL_12ms     0x02
#define AGC_DL_16ms     0x03

/*********************************************************************
**  AMP (Address: 0A02h)                      **
*********************************************************************/
//MA1: power down offset cancellation circuit, active low

//MA0: ADC quantization step.
// 0: 1.5V. 1: 1.8V.
#define MA_1_5V 0x00
#define MA_1_8V 0x40

//[0]: auto AGC gain control. [1]: manual AGC gain control by SA[2:0], SB[1:0] in address 0x0A02h.
#define MAGS  0x20 // Manual AGC gain select.

//SA: MIC PGA coarse setting.
//[000]: 0dB. [001]: 6dB. [010]: 12dB. [011]: 18dB. [100]: 24dB. [101]: 30dB. [11X]: 36dB.
#define SA_0dB  0x00
#define SA_6dB  0x04
#define SA_12dB 0x08
#define SA_18dB 0x0C
#define SA_24dB 0x10
#define SA_30dB 0x14
#define SA_36dB 0x1C
//SB: MIC PGA fine setting.
//[00]: 0dB. [01]: 1.5dB. [10]: 3dB. [11]: 4.5dB.
#define SB_0dB    0x00
#define SB_1_5dB  0x01
#define SB_3dB    0x02
#define SB_4_5dB  0x03


/*********************************************************************
**  PK CAL (Address: 0A03h)                     **
*********************************************************************/
//VTHS: AGC target upper limit. Recommand value= [100].
//VTLS: AGC target lower limit. Recommand value= [010].
// [000]:   0mV. [001]: 320mV. [010]: 480mV. [011]: 600mV.
// [100]: 720mV. [101]: 840mV. [11X]: 1000mV.
#define VTHS_200mV  0x00
#define VTHS_250mV  0x08
#define VTHS_300mV  0x10
#define VTHS_400mV  0x18
#define VTHS_500mV  0x20
#define VTHS_650mV  0x28
#define VTHS_800mV  0x30
#define VTHS_1000mV 0x38

#define VTLS_200mV  0x00
#define VTLS_250mV  0x01
#define VTLS_300mV  0x02
#define VTLS_400mV  0x03
#define VTLS_500mV  0x04
#define VTLS_650mV  0x05
#define VTLS_800mV  0x06
#define VTLS_1000mV 0x07

#define NATEN       0x40

/*********************************************************************
**   OSCAL2 (Address: 0A05h)                    **
*********************************************************************/
#define POLSEL      0x20 // select rising/falling latch from analog ADCdout for digital
//  AC filter's input.

//MOD_CAB: Class AB output current mode, four current driving strengths can be selected
#define MOD_CAB1    0x08
#define MOD_CAB2    0x10
#define MOD_CAB3    0x18

// CAF: Input zeroing for offset calibration.
#define CAF         0x40  // [0]: No zeroing. [1]: Zeroing.
//MOF: Offset adjustment.
#define MOF         0x02  //Adjustment value= C*0.5mV + MOF*100mV.
//SD: Offset ajdustment polarity.
#define SD          0x01  //[0]: Positive. [1]: Negative.

/*********************************************************************
**   ADC_DLY (Address: 0A06h)                     **
*********************************************************************/
//For power up/down ADC delay.
//For power up.
//Tup = 25msX(1+DLUP).
//For power down.
#define DLDN_TD1S_TU25msX      0x00
#define DLDN_TD2S_TU25msX      0x01
#define DLDN_TD3S_TU25msX      0x02
#define DLDN_TD4S_TU25msX      0x03
#define DLDN_TD4_5S_TU25msX    0x04
#define DLDN_TD5S_TU25msX      0x05
#define DLDN_TD5_5S_TU25msX    0x06
#define DLDN_TD6S_TU25msX      0x07


//#define DLDN_TD1S_TU25msX       0x00
#define DLDN_TD2S_TU50msX       0x11
#define DLDN_TD3S_TU75msX       0x22
#define DLDN_TD4S_TU100msX      0x33
#define DLDN_TD4_5S_TU125msX    0x44
#define DLDN_TD5S_TU150msX      0x45
#define DLDN_TD5_5S_TU175msX    0x66
#define DLDN_TD6S_TU200msX      0x77

/*********************************************************************
**   SRSEL (Address: 0A09h)                     **
*********************************************************************/
// SRSEL: sample rate select
#define SRSEL8K     0x00  // 2'b00: sample rate = 8kHz;
#define SRSEL16K    0x40  // SRSEL = 2'b01: sample rate = 16kHz;
#define SRSEL32K    0x80  // SRSEL = 2'b10: sample rate = 32kHz;
#define SRSEL64K    0xC0  // SRSEL = 2'b11: sample rate = 64kHz; (reserved, not

#define I2STX_EN    0x20    // I2STX_EN = 1, I2S TX enable, I2STX_EN = 0, I2S TX disable
#define I2SRX_EN    0x10    // I2SRX_EN = 1, I2S RX enable, I2SRX_EN = 0, I2S RX disable      // ready for this version)
#define ALAW_EN     0x08  //1: alaw encode, ALAW_EN = 0: no alaw encode
#define ULAW_EN     0x04  //1: ulaw encode, ULAW_EN = 0: no ulaw encode
#define ALAW_DE     0x02  //1: alaw decode, ALAW_DE = 0: no alaw decode
#define ULAW_DE     0x01  //1: ulaw decode, ULAW_DE = 0: no ulaw decode


/*********************************************************************
**   CODEC STATE (Address: 0A0Ah)                   **
*********************************************************************/
//This is a read only register for Peak detect & ADC/DAC interrupt state
#define CLR_INT3  0x40  // VOX interrupt flag for detect VOX signal, high active.
#define VTB   0x30  // status from Peak detect
#define VOXIF 0x08  // VOX interrupt flag for detect VOX signal, high active.
// It is inverted than conntect to INT3.
#define VOXSF 0x04  // VOX status flag. When 0, voice is active;
// When 1, voice is off.
//The signals of ADCSTAT and DACSTAT are OR-wired andit is inverted than connected to //INT4. Please check ADCSTAT and DACSTAT, if INT4 occured.
#define ADCSTAT 0x02  // ADCSTAT = 1, INT4 is from ADC; ADCSTAT = 0, INT4 isnot from ADC;
#define DACSTAT 0x01  // DACSTAT = 1, INT4 is from DAC; DACSTAT = 0, INT4 isnot from DAC;

/*********************************************************************
**   CODEC_EFFCTL (Address: 0A0Dh)                    **
*********************************************************************/
#define EFFCTL_EQ   0x10 // Equalizer control, high active.
#define EFFCTL_INTP 0x08 // Interpolation enable, high active.
#define EFFCTL_SIGN 0x04 // sign bit for step size of fade-in or fade-out.
//When SIGN = 0, step is positive. And fade-in is executed if FADE is set to 1.
//When SIGN = 1, step is negative. And fade-out is executed if FADE is set to 1.
#define EFFCTL_FADE 0x02 // Fade control, high active.
#define EFFCTL_MUTE 0x01 // Mute control, high active.


/*********************************************************************
**   CODECTL (Address: 0A0Eh)                     **
*********************************************************************/
#define MPDN_AD      0x80  // manually power down ADC via MCU, low active.
//When ADCIS = 1, interrupt will occur at FIFO full (two voice samples are written in the FIFO).
//When ADCIS = 0, interrupt will occur at FIFO non-empty (one voice sample is written in the FIFO).
#define ADCIS        0x40  // ADC interrupt mode select.
#define ADC_RST      0x20  // digital ADC soft reset, active low.
#define ADCEN        0x10  // enable digital part of ADC, active high.
//When DACIS = 1, interrupt will occur at FIFO empty (there is no voice sample in the FIFO).
//When ADCIS = 0, interrupt will occur at FIFO non-full (less than two voice samples in the FIFO).
#define DACIS        0x08  // DAC interrupt mode select.
#define EXT_EN       0x04  // analog DAC input source select.
#define DAC_RST      0x02  // digital DAC soft reset, active low
#define DACEN        0x01  // enable digital part of DAC, active high.
#define PowerDown    0x00  // PowerDown All circuit
/*********************************************************************
**   TESTMODE (Address: 0A0Fh)                    **
*********************************************************************/
//When CTPS = 1, the test pins are for CODEC.
//When CTPS = 1, the test pins are for baseband. The test pins are Port 0.7, Port 1.2, and Port 1.3.
#define CTPS      0x80  // Codec Test Pin Select.
#define DACRP     0x40  // DAC read pointer indicator.
#define ADCWP     0x20  // ADC write pointer indicator.
#define DACIE     0x10  // DAC interrupt enable, high active.
#define ADCIE     0x08  // ADC interrupt enable, high active.
#define DACSRSEL  0x04    // Digital DAC input source select. 0: from MCU, 1: from I2S.
// ADCSRSEL = 0: ADC(digital) source is from internal analog ADC.
// ADCSRSEL = 1: ADC(digital) source is from external input (I/O pad).
#define ADCSRSEL  0x02  // digital ADC input source select
//ADCTEST: ADC test mode,
//ADCTEST = 0: bbgio are for I2S in/out
//ADCTEST = 1: bbgio are for analog ADC out & DAC pwmout or ext_din for digital ADC
//The test mode is shown as follows,
#define ADCTEST   0x01
#define ADDATEST  0x03
/*********************************************************************
**   PIN Define
*********************************************************************/
#define GIO1      P0_7
#define GIO2      P1_2
#define CKO       P1_3
#endif