#define _FUNCTION_C_

#include <absacc.h>
#include <intrins.h>
#include "..\include\define.h"
#include "..\include\A9101.h"
#include "..\include\main.h"
#include "..\include\globalVar.h"
#include "..\include\Lib_CopyFunction.h"
#include "..\include\Lib_RF_Function.h"

#define ID1 0x80                        // ID1 bit - sets inc/dec opt. for DPTR1
#define ID0 0x40                        // ID0 bit - sets inc/dec opt. for DPTR0
#define TSL 0x20                        // TSL bit - sets toggle function
#define AU  0x10                        // AU1 bit - sets auto inc/dec for DPTR / DPTR1
#define SEL 0x01                        // SEL bit - Select active data pointer

/**************************************************************************************************/
#define ADCINP32  0                     /*EXT ADC input PIN P3_2*/
#define ADCINP33  1                     /*EXT ADC input PIN P3_3*/
#define ADCINP34  2                     /*EXT ADC input PIN P3_4*/
#define ADCINP35  3                     /*EXT ADC input PIN P3_5*/
/**************************************************************************************************/

#define Pair1_IO P0_0
#define Pair2_IO P0_1
#define Pair_IO  P0_3

#define RSSI_threshold	150

/**************************************************************************************************
 * Function Name :                                                                                *
 *       void SARADC12B(Uint8 InputPin)                                                       *
 *                                                                                                *
 * Description:                                                                                   *
 *       Initial 12bit SAR ADC setting                                                            *
 *       note: ADC MODE -> Continuous mode                                                        *
 *                                                                                                *
 * Parameters:                                                                                    *
 *        [INPUT] InputPin                                                                        *
 *        -ADCINP32  0   //EXT ADC input PIN P3_2                                                 *
 *        -ADCINP33  1   //EXT ADC input PIN P3_3                                                 *
 *        -ADCINP34  2   //EXT ADC input PIN P3_4                                                 *
 *        -ADCINP35  3   //EXT ADC input PIN P3_5                                                 *
 * Returns:                                                                                       *
 *        Read 12bit SAR ADC Value                                                                                   *
 **************************************************************************************************/
Uint16 SAR12BADC ( Uint8 InputPin )
{
	  Uint16 MVADC_value;
		Uint8 tmp;
	
    StrobeCmd ( CMD_STBY );
    IOSEL |= 0x20;            /*enable ADCIOS*/

    switch ( InputPin )
    {
        case 0:
            IOSEL |= 0x00;    /*ADC0=P3.2*/
            P3OE &= ~0x04;    /*P3.2 as input*/
            break;

        case 1:
            IOSEL |= 0x40;    /*ADC1=P3.3*/
            P3OE &= ~0x08;    /*P3.3 as input*/
            break;

        case 2:
            IOSEL |= 0x80;    /*ADC2=P3.4*/
            P3OE &= ~0x10;    /*P3.4 as input*/
            break;

        case 3:
            IOSEL |= 0xC0;    /*ADC3=P3.5*/
            P3OE &= ~0x20;    /*P3.4 as input*/
            break;
    }

    A9101_WriteReg ( ADC_REG, A9101_CONFIGTABLE[ADC_REG - CONFIG] | 0x04 ); /*set XADS*/
	  A9101_WriteReg(ADCCTL_REG, 0x07);		//CKS=0, MVS=3, single mode, ADCE=1
		
		EA = 1;

		do{
			tmp = A9101_ReadReg(ADCCTL_REG);  	//wait ADC conversion complete
		}while(tmp & 0x01);
		
    MVADC_value = ( A9101_ReadReg ( ADCAVG1_REG ) & 0xF0 );
    MVADC_value <<= 4;
    MVADC_value = MVADC_value + A9101_ReadReg ( ADCAVG2_REG );
		
    return MVADC_value;
}

Uint8 Read_RXIndex ( Uint8 Index )
  {
    Uint8 i, RSSI;

    IOSEL = 0x00;
    GIO1 = 0;
    GIO1 = 1;

    RSSI = Index;
    GIO1 = 0;
    GIO2 = 0;
    CKO = 0;

    GIO2 = 1;
    _nop_();
    _nop_();
    for ( i = 0; i < 8; i++ )
    {
      if ( RSSI & 0x80 )
        GIO1 = 1;
      else
        GIO1 = 0;

      _nop_();
      CKO = 1;
      _nop_();
      CKO = 0;
      RSSI = RSSI << 1;
    }
    _nop_();
    _nop_();
    GIO2 = 0;

    GIO1 = 0;
    GIO2 = 0;
    CKO = 0;
    IOSEL = 0x08;
    return RSSI;
  }
	
/****************************************
*InitMCU        *
****************************************/
void InitMCU ( void )
{
  P0    = 0xFF;
  P0OE  = 0xE4;
  P0PUN = 0xE4;
  P0WUN = 0xFF;

  P1    = 0xFF;
  P1OE  = 0x4C;
  P1PUN = 0x4C;
  P1WUN = 0xF8;

  P3    = 0xFF;
  P3OE  = 0xF3; 
  P3PUN = 0xF3; 
  P3WUN = 0xFF; 

  IOSEL = 0x08;
}

/****************************************
*SetInterrupt       *
*IE(A8h)        *
****************************************/
void SetInterrupt ( void )
{
  IE  |= 0x80;
  EIE |= 0x80;
  IP  |= 0x00;
  EIP |= 0x00;
}



/************************************************************************
**  InitTimer0
************************************************************************/
void InitTimer0 ( void )
{
  TR0   = 0;
  TMOD  = ( TMOD & 0xF0 ) | 0x01;   //Timer0 mode1
  TH0   = ( 65536 - t0hrel ) >> 8;  //setup Timer0 high byte, low byte
  TL0   = 65536 - t0hrel;
  TF0   = 0;                        //Clear any pending Timer0 interrupts
  ET0   = 1;                        //Enable Timer0 interrupt
}


/****************************************
*Scan MODE                              *
*Pin_Mode = 1 Slave                     *
*Pin_Mode = 0 Master                    *
****************************************/
Uint8 ScanMode ( bit BTN )
{
//BTN=1=SLAVE
//BTN=0=MASTER
  if ( BTN )
  {
    if ( BTN )
      return SLAVE;
    else
      return NonValid;
  }

  else
  {
    if ( ~BTN )
      return MASTER;
    else
      return NonValid;
  }
}
/****************************************
*A9101_Pair                             *
****************************************/
void A9101_Pair(void)
{  
	Uint8 i, link_OK, pair_cmd;
	Uint8 Slave_rssi;

	while(Pair_IO==0)// Define P0_3
	{
		WriteID(Broadcast_ID_Tab);
		
		if(Pair1_IO == 0 || Pair2_IO == 0)//Define P0_0 & P0_1
		{
//		 	if(P0_2 == 0)
//				Pair1_OK = 0;
//			else if(P0_2 == 0)
//				Pair2_OK = 0;
				Slave_rssi = 0;
				link_OK = 0;
    		A9101_WriteReg (ADC_REG, 0xF2);                                             //RRS=1, FSARS=0,RADC=1,HDM=1 ,RSM=3
    		A9101_WriteReg (MODEC2_REG, A9101_CONFIGTABLE[MODEC2_REG - CONFIG] | 0x40 ); //ARRSI=1
		    EA = 1;                                                                //enable interrupt
		    ERFINT = 1;
		    A9101_WriteReg (FIFO1_REG, 0x01);         								//FIFO Length = 1 
				
			ResetCmd(TXPOINT_RST);   
			if(Pair1_IO == 0)               
				A9101_WriteReg(TX_FIFO, 0x01);
			else if(Pair2_IO == 0) 
				A9101_WriteReg(TX_FIFO, 0x02);
		
			if(P3_2 == 0)	//Master
			{				
				for (i=0;i<1;i++)
				{
					Delay1ms(5);
					A9101_WriteReg(INTSW_REG, 0x02);	//WTR                                			                                      
					SetCH (75);
					RF_FLAG = 1;  	
					StrobeCmd (CMD_TX);                       					                             
					while (RF_FLAG);                                       
					
					StrobeCmd (CMD_STBY);                      //Entry PLL MODE
					Delay1ms(1);
					ResetCmd(RXPOINT_RST);										
			        TIMEOUT = 1 ;
			        t0hrel   = 11200;                            
			        InitTimer0();
			        StrobeCmd(CMD_RX);                        //Entry RX MODE
			        RF_FLAG = 1;                              //Set RF Interrupt Flag
			        TR0 = 1;                                  //Enable Timer0 for counting RX Time out
			        TimeoutFlag = 0;                          //Reset RX Timeout Flag
			        timer = 0;                                //Reset Timer0 counter
			        while (RF_FLAG && (TimeoutFlag == 0));   //Wait for RX Receive done or RX Time out
		
			        if (TimeoutFlag)
			        {
			        	StrobeCmd(CMD_STBY);                    //Entry StandBy MODE
			        }
			        /* RX Receive done Process */
			        else
			        {					
						pair_cmd = A9101_ReadReg(RX_FIFO);
						Slave_rssi = A9101_ReadReg(RX_FIFO+1);
						link_OK++;					 
					}		
				}
			}
			else
			{
				for (i=0;i<1;i++)
				{
					A9101_WriteReg(INTSW_REG, 0x80); 			//Fsync			
 			        SetCH (75);
					RF_FLAG = 1;                              //Set RF Interrupt Flag                         
			        StrobeCmd(CMD_RX);                        //Entry RX MODE

			        while (RF_FLAG && (Pair_IO == 0));   		  //Wait for RX Receive					

					if((RF_FLAG == 0) && (Pair_IO == 0))
					{	
						Slave_rssi = A9101_ReadReg(RSSI_REG);
						RF_FLAG = 1; 
						A9101_WriteReg(INTSW_REG, 0x02);	//WTR 						
						while (RF_FLAG && (Pair_IO == 0));   		  //Wait for RX Receive done 
												
						link_OK++;
						pair_cmd = A9101_ReadReg(RX_FIFO);						
	
						A9101_WriteReg(TX_FIFO+1, Slave_rssi);

						Delay1ms(5);
						SetCH (75);
						A9101_WriteReg(INTSW_REG, 0x02);	//WTR                                			                                      						
						RF_FLAG = 1;  	
						StrobeCmd(CMD_TX);                       					                             
						while (RF_FLAG);  
					}
					else
					{
						link_OK = 0;					
					}			
			
			  }
			
			}

			if(link_OK && Slave_rssi > RSSI_threshold)
			{
				if(Pair1_IO == 0 && pair_cmd == 0x01)
					Pair1_OK = 1;
				else if(Pair2_IO == 0 && pair_cmd == 0x02)
					Pair2_OK = 1;
				else
					break;			
			
				while(Pair_IO == 0)	//pair ok, user can print message...
					P0_2 ^=1;
			}		
			else
			{
				//pair fail, user can print message...
			
			}
		}
	}
}
/****************************************
*A9101_Battery_Detect                   *
****************************************/
Uint8 A9101_Battery_Detect(void)
{   
	Uint8 tmp;

//	A9101_WriteReg(BATTERY_REG, 0x01);	//0, 2.1V
//	A9101_WriteReg(BATTERY_REG, 0x03);	//1, 2.2V
//	A9101_WriteReg(BATTERY_REG, 0x05);	//2, 2.3V
//	A9101_WriteReg(BATTERY_REG, 0x07);	//3, 2.4V
//	A9101_WriteReg(BATTERY_REG, 0x09);	//4, 2.5V
//	A9101_WriteReg(BATTERY_REG, 0x0B);	//5, 2.6V
//	A9101_WriteReg(BATTERY_REG, 0x0D);	//6, 2.7V
	A9101_WriteReg(BATTERY_REG, 0x0F);	//7, 2.8V
			
	Delay100us(1);

	tmp = A9101_ReadReg(BATTERY_REG) & 0x10;

	return tmp;
}
/****************************************
*Channel_mode                           *
****************************************/
void Channel_mode(void)
{
  Uint8 i;

  if(P3_3 == 0)		//single channel mode
  {
		for ( i = 0; i < 4; i++ )
		{  
			HoppingCH[0][i] = 85;	//488.721 MHz
				HoppingCH[1][i] = 85;
				HoppingCH[2][i] = 85;
			HoppingCH[3][i] = 85;
		}
  }
  else
  {
		HoppingCH[0][0] =5;		//468.241 MHz
		HoppingCH[0][1] =35;	//475.921 MHz
		HoppingCH[0][2] =25;	//473.361 MHz
		HoppingCH[0][3] =15;	//470.801 MHz			
		HoppingCH[1][0] =10;	//469.521 MHz
		HoppingCH[1][1] =20;	//472.801 MHz
		HoppingCH[1][2] =30;	//474.641 MHz
		HoppingCH[1][3] =40;	//477.201 MHz	
		HoppingCH[2][0] =45;	//478.481 MHz
		HoppingCH[2][1] =75;	//486.161 MHz
		HoppingCH[2][2] =65;	//483.601 MHz
		HoppingCH[2][3] =55;	//481.041 MHz
		HoppingCH[3][0] =50;	//479.761 MHz
		HoppingCH[3][1] =60;	//482.321 MHz
		HoppingCH[3][2] =70;	//484.881 MHz
		HoppingCH[3][3] =80;	//487.441 MHz  
  } 
}
/************************************************************************
**  WriteID
************************************************************************/
Uint8 WriteID(Uint8* ID_Tab)
{
	Uint8 i;
    Uint8 xdata d[8];

	for(i=0; i<8; i++)
		A9101_WriteReg((ID0_REG - i), ID_Tab[i]);

    //Read ID and check
    for(i=0;i<8;i++)
    {
        d[i] = A9101_ReadReg(ID0_REG - i);
  		if(ID_Tab[i]!=d[i])
    	    return FAIL;
    }

	return PASS;
}
/****************************************
*Device Link                            *
****************************************/
bit DeviceLink ( Uint8 MODE )
{
  Uint8 i, VCOSC, RGVD;
  Uint8 LinkFlag;
  Uint8 LossCnt;
  Uint8 GetPacket;
  Uint8 TXTimes = 0;
  Uint8 VOLTest = 0;
  Uint8 FRETest = 0;
  Uint8 cmd[4];
  Uint8 channel_set=0;
  bit broacast_flag = 0;

  LinkFlag = 1;
  LossCnt = 0;
  GetPacket = 0;
  VOLTest=0x3C;
  if ( MODE == MASTER )
  {
		A9101_WriteReg ( GIO1_REG, 0x3C ); //FMTDO (FIFO mode TX Data Output testing)
		A9101_WriteReg ( GIO2_REG, 0x00 ); //WTR
		A9101_WriteReg ( CKO_REG,  0x00 ); //DCk
	}
	else
	{
		A9101_WriteReg ( GIO1_REG, 0x1C ); //PDN_RX
		A9101_WriteReg ( GIO2_REG, 0x00 ); //WTR
		A9101_WriteReg ( CKO_REG,  0x00 ); //RCK
	}

  for ( i = 0; i < 96; i++ )
  { 
    CODECBUF[0][i] = 0x00;
    CODECBUF[1][i] = 0x00;
  }

  /***********************/
  /*       MASTER        */
  /***********************/
  if ( MODE == MASTER )
  {
	/* Set RF */
    EA = 1;                                                                //enable interrupt
    A9101_WriteReg ( INTSW_REG, 0x02 );                                    //enable WTR interrupt
    A9101_WriteReg ( RX_REG,  A9101_CONFIGTABLE[RX_REG - CONFIG] | 0x80 ); //enable MSCRC
    A9101_WriteReg ( FIFO1_REG, 0x63 );         //FIFO Length = 96 bytes + 4byte cmd
    /* Set RSSI */
    A9101_WriteReg ( ADC_REG, 0xF2 );                                             //RRS=1, FSARS=0,RADC=1,HDM=1 ,RSM=3
    A9101_WriteReg ( MODEC2_REG, A9101_CONFIGTABLE[MODEC2_REG - CONFIG] | 0x40 ); //ARRSI=1
    /* Set CODEC */
    TxIndex = 0;
    TXBufIndex = 0;
    DataTxFlag = 0;
    InitCodec ( MASTER );
    A9101_WriteReg ( CODEC_CODECTL, 0xB0 );       //ADC ON

    StrobeCmd ( CMD_PLL );
    Delay1ms ( 1 );
    while ( MODE == MASTER )
    {
			Channel_mode();
		
			if(Pair1_OK && (Pair1_IO == 0))
				WriteID(Pair1_ID_Tab);
			else if(Pair2_OK && (Pair2_IO == 0))	
				WriteID(Pair2_ID_Tab);
			else
				WriteID(Broadcast_ID_Tab);
					
			while(~DataTxFlag);                      //Wait ADC Convert done

				/* Write Packet to FIFO Buffer */
			ResetCmd (TXPOINT_RST);                    //Reset RX FIFO Point
			X2X_DDP_MAMCPY(AU+TSL,CODECBUF[~TxIndex],TX_FIFO,96);	//voice data
			A9101_WriteReg(TX_FIFO+96, 0x01);						//control data
			A9101_WriteReg(TX_FIFO+97, 0x02);
			A9101_WriteReg(TX_FIFO+98, 0x03);
			A9101_WriteReg(TX_FIFO+99, 0x04);
       
      
      for ( TXTimes = 0; TXTimes < 4; TXTimes++ )
      {
      	Delay100us(1);
      	
        /* Send TX */
        ERFINT = 1;                                 //Enable RF Interrupt
        SetCH (HoppingCH[TXTimes][channel_set]);	//Set RF Channel
        StrobeCmd ( CMD_TX );                       //Entry TX MODE
        RF_FLAG = 1;                                //Set RF Interrupt Flag
        while ( RF_FLAG );                          //Waiting RF Interrupt Trigger
				Read_RXIndex ( TXBufIndex );               //For Audio   0x1C=28 28/2 =14 ,14*125us = 1.75ms  1.769ms
        StrobeCmd ( CMD_PLL );                      //Entry PLL MODE
      }

			channel_set++;

			if(channel_set == 4)
				channel_set	= 0;	  	

      if ( ScanMode ( Pin_ModeSelect ) != MASTER || ScanMode ( Pin_ModeSelect ) == NonValid )
        return 0;

			if(Pair_IO == 0)
			{
				A9101_WriteReg(CODEC_CODECTL, 0x00);       //ADC OFF
				return 0;
			}
				DataTxFlag = 0;                               //Wait ADC Convert done
		}
  }
  /***********************/
  /*       SLAVE        */
  /***********************/
  else
  {
		/* Set RSSI */
    A9101_WriteReg ( ADC_REG, 0xF2 );                                             //RRS=1, FSARS=0,RADC=1,HDM=1 ,RSM=3
    A9101_WriteReg ( MODEC2_REG, A9101_CONFIGTABLE[MODEC2_REG - CONFIG] | 0x40 ); //ARRSI=1
    /* Set RF */
    EA = 1; //enable interrupt
    A9101_WriteReg ( INTSW_REG, 0x02 );                                    //enable WTR interrupt
    A9101_WriteReg ( RX_REG,  A9101_CONFIGTABLE[RX_REG - CONFIG] | 0x80 ); //enable MSCRC
  	A9101_WriteReg ( FIFO1_REG, 0x63 );         //FIFO Length = 96 bytes + 4byte cmd
    /* Set CODEC */
    Empty      = 1;                              //Set Empty Flag
    RxIndex    = 0;
    TXBufIndex = 0;
    RXBufIndex = 0;
    InitCodec ( SLAVE );
    A9101_WriteReg ( CODEC_CODECTL, 0x03 );      //DAC ON
    A9101_WriteReg ( CODEC_EFFCTL, 0x01 );       //Set Mute
    
    A9101_WriteReg(CODEC_VOL, VOLTest);

    GetPacket = 0;
    while ( GetPacket == 0 )
    {     	
			VCOSC = 56;
			RGVD = A9101_ReadReg(EXT1_REG);
			A9101_WriteReg(EXT1_REG, RGVD | VCOSC);
			Channel_mode(); 

			if(Pair1_OK && (Pair1_IO == 0))
			{
				if(broacast_flag)
					WriteID(Pair1_ID_Tab);
				else
					WriteID(Broadcast_ID_Tab);
			}
			else if(Pair2_OK && (Pair2_IO == 0))	
			{
				if(broacast_flag)
					WriteID(Pair2_ID_Tab);
				else
					WriteID(Broadcast_ID_Tab);
			}
			else
				WriteID(Broadcast_ID_Tab);

			broacast_flag ^=1;//broacast_flag=1
	  
			if ( ScanMode ( Pin_ModeSelect ) != SLAVE || ScanMode ( Pin_ModeSelect ) == NonValid )
				return 0;
	 
			if(Pair_IO == 0)
				return 0; 
				
			for ( TXTimes = 0; TXTimes < sizeof 4; TXTimes++ )
			{
        if ( GetPacket )
          break;

        TIMEOUT = 1 ;
        t0hrel   = 22400;                            //2.03ms @16.384MHz
        InitTimer0();
        SetCH ( HoppingCH[TXTimes][channel_set]);    //Set RF Channel
        ResetCmd ( RXPOINT_RST );                    //Reset RX FIFO Point
        ERFINT = 1;                                  //Enable RF Interrupt
        StrobeCmd ( CMD_RX );                        //Entry RX MODE
        RF_FLAG = 1;                                 //Set RF Interrupt Flag
        TR0 = 1;                                     //Enable Timer0 for counting RX Time out
        TimeoutFlag = 0;                             //Reset RX Timeout Flag
        timer = 0;                                   //Reset Timer0 counter
        while ( RF_FLAG && ( TimeoutFlag == 0 ) );   //Wait for RX Receive done or RX Time out
        if ( TimeoutFlag )
        {
          ERFINT = 0;                                //Disable RF Interrupt
          StrobeCmd ( CMD_STBY );                    //Entry StandBy MODE
        }
        /* RX Receive done Process */
        else
        {
					Read_RXIndex ( RXBufIndex );            //for 
          GetPacket = 1;
          switch ( TXTimes )
          {
          case 0:
					  RXBufIndex = 0x15 ;
            break;
          case 1:
					  RXBufIndex = 0x2B; 
            break;
          case 2:
					  RXBufIndex = 0x40 ;
            break;
          case 3:
					  RXBufIndex = 0x57 ;
            break;
          }
        }
      }
    }

    while ( Empty ); //Wait for DAC Buffer Empty
    Empty = 1;
    while ( Empty ); //Wait for DAC Buffer Empty
    Empty = 1;  

		channel_set = 1;  
			
		while ( MODE == SLAVE )
		{
      GetPacket = 0;
      
      for ( TXTimes = 0; TXTimes < 4; TXTimes++ )
      {
        switch ( TXTimes )
        {
					case 0:
						TIMEOUT = 1 ;
						t0hrel   = 3900;                             //2.03ms @16.384MHz
						InitTimer0();
						break;
					case 1:
						TIMEOUT = 1 ;
						t0hrel   = 3700;                             //2.03ms @16.384MHz
						InitTimer0();
						break;
					case 2:
						TIMEOUT = 1 ;
						t0hrel   = 3700;                             //2.03ms @16.384MHz
						InitTimer0();
						break;
					case 3:
						TIMEOUT = 1 ;
						t0hrel   = 4000;                             //2.03ms @16.384MHz
						InitTimer0();
						break;
        }
        
        SetCH (HoppingCH[TXTimes][channel_set]);		//Set RF Channel
        ResetCmd ( RXPOINT_RST );                      //Reset RX FIFO Point
        ERFINT = 1;                                    //Enable RF Interrupt
        StrobeCmd ( CMD_RX );                          //Entry RX MODE
        RF_FLAG = 1;                                   //Set RF Interrupt Flag
        TR0 = 1;                                       //Enable Timer0 for counting RX Time out
        TimeoutFlag = 0;                               //Reset RX Timeout Flag
        timer = 0;                                     //Reset Timer0 counter
        while ( RF_FLAG && ( TimeoutFlag == 0 ) );     //Wait for RX Receive done or RX Time out
        TR0 = 0;                                       //Disable Timer0
        TIMEOUT = 1 ;
        /* RX Time out Process */
        if ( TimeoutFlag )
        {
          ERFINT = 0;                                  //Disable RF Interrupt
          StrobeCmd ( CMD_STBY );                      //Entry StandBy MODE
          LossCnt ++;
					if(TXTimes == 3)
					{
						P0_2 = 0;
						P0_2 = 1;
						P0_2 = 0;				
					}
        }
        /* RX Receive done Process */
        else
        {
          Read_RXIndex ( RXBufIndex );  
					switch ( TXTimes )
					{
						case 0:
							if ( RXBufIndex <= 0x12) 
							{
								VCOSC = VCOSC - 1 ;
								A9101_WriteReg(EXT1_REG, RGVD | VCOSC);
							}
							if ( RXBufIndex >= 0x17)
							{
								VCOSC = VCOSC + 1 ;
								A9101_WriteReg(EXT1_REG, RGVD | VCOSC);
							}	
							break;

						case 1:
							if ( RXBufIndex <= 0x27) 
							{
								VCOSC = VCOSC - 1 ;
								A9101_WriteReg(EXT1_REG, RGVD | VCOSC);
							}
							if ( RXBufIndex >= 0x2F)
							{
								VCOSC = VCOSC + 1 ;
								A9101_WriteReg(EXT1_REG, RGVD | VCOSC);
							}
							break;

						case 2:
							if ( RXBufIndex <= 0x3C) 
							{
								VCOSC = VCOSC - 1 ;
								A9101_WriteReg(EXT1_REG, RGVD | VCOSC);
							}
							if ( RXBufIndex >= 0x44)
							{
								VCOSC = VCOSC + 1 ;
								A9101_WriteReg(EXT1_REG, RGVD | VCOSC);
							}								
							break;

						case 3:
							if ( RXBufIndex <= 0x53) 
							{
								VCOSC = VCOSC - 1 ;
								A9101_WriteReg(EXT1_REG, RGVD | VCOSC);
							}
							if ( RXBufIndex >= 0x5B)
							{
								VCOSC = VCOSC + 1 ;
								A9101_WriteReg(EXT1_REG, RGVD | VCOSC);
							}
							break;
          }
          LossCnt = 0 ;
          GetPacket = 1;
        }
        if ( GetPacket )
          break;
      }
      GetPacket = 0;

      if ( LossCnt > 10 )
        return 0;

			channel_set++;

			if(channel_set == 4)
				channel_set = 0;
					
			while ( Empty ); //Wait for DAC Buffer Empty

			X2X_DDP_MAMCPY(AU+TSL,RX_FIFO,CODECBUF[~RxIndex],96);  
			cmd[0]=A9101_ReadReg(RX_FIFO+96); 
			cmd[1]=A9101_ReadReg(RX_FIFO+97); 
			cmd[2]=A9101_ReadReg(RX_FIFO+98); 
			cmd[3]=A9101_ReadReg(RX_FIFO+99); 
		
      if ( LossCnt == 0 )
			{
				FRETest=A9101_ReadReg(PLL1_REG);
      	
				if(P1_6==0)
				{
					A9101_WriteReg ( CODEC_EFFCTL, 0x01 );//Mute
				}
				else
				{
	        A9101_WriteReg ( CODEC_EFFCTL, 0x00 );//Disable Mute	        
					if(P1_1==0)
					{
						if(VOLTest>=0x3C)
							VOLTest=0x3C;
						else
							VOLTest=VOLTest+1;
						A9101_WriteReg(CODEC_VOL, VOLTest);
					}
					else if(P1_0==0)
					{
						if(VOLTest<=0x00)
							VOLTest=0;
						else
							VOLTest=VOLTest-1;

						A9101_WriteReg(CODEC_VOL, VOLTest);	
		      }		       
					A9101_WriteReg ( CODEC_EFFCTL, 0x00 );//Disable Mute
	      }
			}
			else
			{
				A9101_WriteReg ( CODEC_EFFCTL, 0x01 );//Mute
			}

			Empty = 1; //Set Empty Flag to avoid DAC Buffer doesn't finish the play

			if ( ScanMode ( Pin_ModeSelect ) != SLAVE || ScanMode ( Pin_ModeSelect ) == NonValid ||Pair_IO == 0)
        return 0;
    }
  }
  return 1;
}
